#!/usr/bin/env python
#
# Copyright (C) 2011 Ursula Junque <ursinha@ursinha.net>
# Copyright (C) 2011 Canonical Ltd
# Copyright (C) 2012 Linaro Ltd
#
# Author: Ursula Junque <ursinha@ursinha.net>
#         Ricardo Salveti <rsalveti@rsalveti.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from launchpadlib.launchpad import Launchpad
import reporting
from datetime import datetime, timedelta

class Bug:

    def __init__(self, lp_bug):
        self._bug_id = None
        self._title = None
        self._tags = None
        self._associated_projects = None
        self.lp_bug = lp_bug

    @property
    def title(self):
        if self._title is None:
            self._title = self.lp_bug.title
        return self._title

    @property
    def bug_id(self):
        if self._bug_id is None:
            self._bug_id = self.lp_bug.id
        return self._bug_id

    @property
    def tags(self):
        if self._tags is None:
            self._tags = self.lp_bug.tags
        return self._tags

    @property
    def associated_projects(self):
        if self._associated_projects is None:
            self._associated_projects = [task.bug_target_name 
                    for task in self.lp_bug.bug_tasks] 
        return self._associated_projects


class BugTask:

    def __init__(self, lp_bugtask):
        self.lp_bugtask = lp_bugtask
        self.activities = list(lp_bugtask.bug.activity)
        self._date_created = None
        self.bug = Bug(lp_bugtask.bug)
        self._bug_id = None
        self._assignee = None
        self._importance = None
        self._status = None
        self._target = None

        self.activities.reverse()
        for activity in self.activities:
            if "subscrib" in activity.whatchanged:
                continue
            elif activity.message is not None and "subscrib" in activity.message:
                continue
            break
        self.lastactivity = Activity(activity)
        self.age_days = calculateDays(self.date_created)
        self.age_hours = calculateHours(self.date_created) % 24
        self.last_touched_days = calculateDays(self.lastactivity.datechanged)
        self.last_touched_hours = calculateHours(self.lastactivity.datechanged)
        self.last_touched_minutes = calculateMinutes(self.lastactivity.datechanged)

    @property
    def date_created(self):
        if self._date_created is None:
            self._date_created = self.lp_bugtask.date_created
        return self._date_created

    @property
    def bug_id(self):
        if self._bug_id is None:
            self._bug_id = self.bug.bug_id
        return self._bug_id

    @property
    def assignee(self):
        if self._assignee is None:
            self._assignee = self.lp_bugtask.assignee
        return self._assignee

    @property
    def importance(self):
        if self._importance is None:
            self._importance = self.lp_bugtask.importance
        return self._importance

    @property
    def status(self):
        if self._status is None:
            self._status = self.lp_bugtask.status
        return self._status

    @property
    def target(self):
        if self._target is None:
            self._target = self.lp_bugtask.target.name
        return self._target


def calculateDays(date):
    if date != "-":
        t = datetime.utcnow()
        try:
            now = datetime.strptime(date.split(".")[0], "%Y-%m-%d %H:%M:%S")
        except AttributeError:
            # date is a datetime object already
            now = date
            t = t.replace(tzinfo=now.tzinfo)
        age = t - now
        days = age.days
    else:
        days = 0
    return int(days)


def calculateHours(date):
    if date != "-":
        t = datetime.utcnow()
        try:
            now = datetime.strptime(date.split(".")[0], "%Y-%m-%d %H:%M:%S")
        except AttributeError:
            # date is a datetime object already
            now = date
            t = t.replace(tzinfo=now.tzinfo)
        age = t - now
        seconds = int(age.seconds)
        hours = seconds/3600
    else:
        hours = 0
    return int(hours)


def calculateMinutes(date):
    if date != "-":
        t = datetime.utcnow()
        try:
            now = datetime.strptime(date.split(".")[0], "%Y-%m-%d %H:%M:%S")
        except AttributeError:
            # date is a datetime object already
            now = date
            t = t.replace(tzinfo=now.tzinfo)
        age = t - now
        seconds = int(age.seconds)
        minutes = seconds/60
    else:
        minutes = 0
    return int(minutes)


class Activity:
    def __init__(self, lp_activity, whatchanged=None, datechanged=None,
            person=None, message=None, oldvalue=None, newvalue=None):
        self._whatchanged = whatchanged
        self._datechanged = datechanged
        self._person = person
        self._message = message
        self._oldvalue = oldvalue
        self._newvalue = newvalue
        self.activity = lp_activity
        self.propertychanged = ""
        self.diff = ""
        self.parse_changes()

    @property
    def whatchanged(self):
        if self._whatchanged is None:
            self._whatchanged = self.activity.whatchanged
        return self._whatchanged

    @property
    def datechanged(self):
        if self._datechanged is None:
            self._datechanged = self.activity.datechanged
        return self._datechanged

    @property
    def person(self):
        if self._person is None:
            self._person = self.activity.person
        return self._person

    @property
    def message(self):
        if self._message is None:
            self._message = self.activity.message
        return self._message

    @property
    def oldvalue(self):
        if self._oldvalue is None:
            self._oldvalue = self.activity.oldvalue
        return self._oldvalue

    @property
    def newvalue(self):
        if self._newvalue is None:
            self._newvalue = self.activity.newvalue
        return self._newvalue

    def parse_changes(self):
        self.propertychanged = self.whatchanged

        if ("importance" in self.whatchanged or
            "status" in self.whatchanged or
            "assignee" in self.whatchanged or
            "milestone" in self.whatchanged):
            pkg_name, whatchanged = self.whatchanged.split(":")
            self.propertychanged = whatchanged.strip()
            self.diff = "%s: %s->%s" % (pkg_name, self.oldvalue, self.newvalue)
        elif "summary" in self.whatchanged:
            self.propertychanged = "summary"
            self.diff = self.newvalue
        elif "tags" in self.whatchanged:
            self.propertychanged = "tags"
            old_tags = self.oldvalue.split()
            new_tags = self.newvalue.split()
            added = []
            removed = []
            for tag in new_tags:
                if tag not in old_tags:
                    added.append(tag)
            for tag in old_tags:
                if tag not in new_tags:
                    removed.append(tag)
            if len(added) > 0:
                self.diff = "added: %s " % ",".join(added)
            if len(removed) > 0:
                self.diff += "removed: %s" % ",".join(removed)
        elif "bug" == self.whatchanged:
            if self.message is not None and "added bug" in self.message:
                self.propertychanged = "bug"
                self.diff = "Bug created"
        elif "attachment added" in self.whatchanged:
            self.diff = self.newvalue
        elif "attachment removed" in self.whatchanged:
            self.diff = self.oldvalue
        elif "branch linked" in self.whatchanged:
            self.diff = self.newvalue
        elif "branch unlinked" in self.whatchanged:
            self.diff = self.oldvalue
        elif "nominated for" in self.whatchanged:
            self.diff = self.newvalue
        elif "bug watch added" in self.whatchanged:
            self.diff = self.newvalue
        elif "bug watch removed" in self.whatchanged:
            self.diff = self.oldvalue
        elif "bug task added" in self.whatchanged:
            self.diff = self.newvalue
        else:
            if self.newvalue is not None:
                self.diff = self.newvalue
            elif self.oldvalue is not None:
                self.diff = self.oldvalue
            elif self.message is not None:
                self.diff = self.oldvalue


if __name__ == "__main__":
    lp = Launchpad.login_with("linaro-reports", "production")
    project = lp.projects["linaro-ubuntu"]

    bugtags = project.official_bug_tags
    bugtasks = list(project.searchTasks())
    try:
        all_bugtasks = [BugTask(task) for task in bugtasks]
    except Exception, e:
        print "Failed to generate the bugtask list: ", e
        raise

    # All bug tasks we want as a tab
    ltti_bugtasks = []
    ltsamsung_bugtasks = []
    ltfreescale_bugtasks = []
    ltste_bugtasks = []
    ltarm_bugtasks = []

    # Process all bugtasks
    for bugtask in all_bugtasks:
        if "linaro-landing-team-ti" in bugtask.bug.associated_projects:
            ltti_bugtasks.append(bugtask)
        if "linaro-landing-team-samsung" in bugtask.bug.associated_projects:
            ltsamsung_bugtasks.append(bugtask)
        if "linaro-landing-team-freescale" in bugtask.bug.associated_projects:
            ltfreescale_bugtasks.append(bugtask)
        if "linaro-landing-team-ste" in bugtask.bug.associated_projects or \
                "igloocommunity" in bugtask.bug.associated_projects:
            ltste_bugtasks.append(bugtask)
        if "linaro-landing-team-arm" in bugtask.bug.associated_projects:
            ltarm_bugtasks.append(bugtask)

    reporting.write_linaro_ubuntu_bugs_html_report(all_bugtasks, ltti_bugtasks,
            ltsamsung_bugtasks, ltfreescale_bugtasks, ltste_bugtasks,
            ltarm_bugtasks, directory="/tmp")
#    reporting.write_wiki_output(all_bugtasks, ltti_bugtasks, ltsamsung_bugtasks,
#            ltfreescale_bugtasks, ltste_bugtasks, ltarm_bugtasks,
#            output_file="/tmp/wiki.out")
