#!/usr/bin/env python
#
# Copyright (C) 2011 Ursula Junque <ursinha@ursinha.net>
# Copyright (C) 2011 Canonical Ltd
#
# Author: Ursula Junque <ursinha@ursinha.net>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import with_statement

__metaclass__ = type

from datetime import datetime
import logging
import os
from genshi.template import TemplateLoader

log = logging.getLogger(__name__)

def load_report_template(template_name):
    """Retrieve a Genshi HTML template instance for the deployment report."""
    loader = TemplateLoader(
        os.path.join(os.path.dirname(__file__), 'templates'))
    return loader.load(template_name)


def render_deployment_report(stream, context, template_name):
    """Render the deployment report to the given stream using the given
    context.
    """
    template = load_report_template(template_name)
    template_stream = template.generate(**context)
    stream.write(template_stream.render('html'))

def write_linaro_ubuntu_bugs_html_report(all_bugtasks, ltti_bugtasks, lts_bugtasks,
        ltfsc_bugtasks, ltste_bugtasks, ltarm_bugtasks, directory=None):

    report_timestamp = datetime.utcnow().strftime('%Y-%m-%d %H:%M:%S')
    context = dict(
        all_bugtasks=all_bugtasks,
        ltti_bugtasks=ltti_bugtasks,
        lts_bugtasks=lts_bugtasks,
        ltfsc_bugtasks=ltfsc_bugtasks,
        ltste_bugtasks=ltste_bugtasks,
        ltarm_bugtasks=ltarm_bugtasks,
        report_timestamp=report_timestamp,
        )

    report_name = "linaro-ubuntu-bugs-report.html"

    report_path = os.path.join(directory, report_name)
    with file(report_path, 'w') as reportfile:
        log.info("Writing HTML report to %s", report_path)
        render_deployment_report(reportfile, context,
            'linaro-ubuntu-bugs-report.html')

def write_wiki_output(all_bugtasks, ltti_bugtasks, lts_bugtasks, ltfsc_bugtasks,
                        ltste_bugtasks, ltarm_bugtasks, output_file):
    reportfile = open(output_file, 'w')
    reportfile.write("<<TableOfContents>>\n")

    def write_table(project, bugtasks):
        reportfile.write("= %s =\n" % project)
        reportfile.write("|| Bug # || Title || Tags || Importance || "
                "Status || Assignee || Bug Last Touched || Bug Age ||\n")
        for bugtask in bugtasks:
            assignee = "-"
            if bugtask.assignee:
                assignee = "[[https://launchpad.net/~%s | %s]]" % (
                    bugtask.assignee.name, bugtask.assignee.display_name)

            line = "|| #Bug:%s || %s || %s || %s || %s || %s || %s || %s ||\n" % (
                bugtask.bug_id, bugtask.bug.title,
                ",".join(bugtask.bug.tags), bugtask.importance,
                bugtask.status, assignee, 
                (bugtask.last_touched_days * 24) + bugtask.last_touched_hours,
                (bugtask.age_days * 24) + bugtask.age_hours)
            reportfile.write(line)

    write_table("Linaro-Ubuntu", all_bugtasks)
    write_table("Texas Instruments Landing Team", ltti_bugtasks)
    write_table("Samsung Landing Team", lts_bugtasks)
    write_table("Freescale Landing Team", ltfsc_bugtasks)
    write_table("ST-Ericsson Landing Team", ltste_bugtasks)
    write_table("ARM Landing Team", ltarm_bugtasks)

    reportfile.flush()
    reportfile.close()



